# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyramid_marshmallow']

package_data = \
{'': ['*']}

install_requires = \
['marshmallow>=3.0,<4.0', 'pyramid>=1.7,<2.0']

extras_require = \
{'openapi': ['apispec>=3.0,<4.0', 'PyYAML>=3.10']}

entry_points = \
{'console_scripts': ['generate-spec = '
                     'pyramid_marshmallow.generatespec:generate']}

setup_kwargs = {
    'name': 'pyramid-marshmallow',
    'version': '0.6.0',
    'description': 'Validate request and response data with Marshmallow and optionally generate an OpenAPI spec.',
    'long_description': '# pyramid-marshmallow\n\npyramid-marshmallow is a simple Pyramid plugin that allows you to validate and\nmarshal a JSON HTTP request or response using\n[Marshmallow](http://marshmallow.readthedocs.io/) schemas.  You can then\nleverage this to automatically generate an OpenAPI specification for your API.\n\n> **Version 0.5 required Marshmallow 3.x.  For Marshmallow 2.x, use version\n> 0.4.**\n\n## Basic usage\n\nInstall the project with `pip install pyramid-marshmallow`.\n\nActivate it by adding `config.include(\'pyramid_marshmallow\')` into your config\nfunction or adding `pyramid.includes = pyramid_marshmallow` into your ini file.\n\nTo validate incoming data, set `validate` to a Marshmallow schema in your\n`view_config`.  The request body is parsed as JSON then passed through the\nschema\'s `load` function.  You can access the processed data with\n`request.data`.\n\n```python\nfrom marshmallow import Schema, String\n\n\nclass HelloSchema(Schema):\n    name = String()\n\n\n@view_config(\n    context=Root,\n    name=\'hello\',\n    request_method=\'post\',\n    validate=HelloSchema(),\n)\ndef hello(context, request):\n    return Response(body=\'Hello, {}\'.format(\n        request.data[\'name\']\n    ))\n```\n\nFor GET requests, the URL parameters are passed into the schema.  Value lists\nare not currently supported.\n\nSetting `marshal` in your `view_config` will run the view output through\nmarshmallow (i.e. `Schema.dump`) before going to the renderer.  You probably\nwill want to set the renderer to `json`.\n\n```\n@view_config(\n    context=Root,\n    name=\'hello\',\n    request_method=\'get\',\n    marshal=HelloSchema(),\n    renderer=\'json\',\n)\ndef hello(context, request):\n    name = fetch_name()\n    return {\n        \'name\': name,\n    }\n```\n\n`validate` and `marshal` operate independently, so can be used separately or\ntogether.\n\nAs a convenience, you can pass in a dictionary to `validate` or `marshal` and\npyramid-marshmallow will turn it into a schema for you.\n\n```python\n@view_config(\n    context=Root,\n    name=\'hello\',\n    request_method=\'post\',\n    validate={\n        \'name\': String(),\n    },\n)\n```\n\nYou can also get a schema made from a dictionary by using the\n`pyramid_marshmallow.make_schema` function.  This can be useful for `Nested`\nfields.\n\n\n### Error handling\n\npyramid-marshmallow passes through exceptions from marshmallow.  So errors\nduring validation will raise a `marshmallow.exceptions.ValidationError`\nexception.\n([Documentation](https://marshmallow.readthedocs.io/en/stable/api_reference.html#marshmallow.exceptions.ValidationError))\n\nYou may want to attach a view to this exception to expose the error messages to\nthe user.\n\n```python\n@view_config(\n    context=ValidationError,\n    renderer=\'json\',\n)\ndef validation_error(context, request):\n    request.response.status = 401  # HTTP Bad Request\n    return {\n        \'errors\': context.normalized_messages(),\n    }\n```\n\n## OpenAPI\n\nBy adding validation and marshalling to your views, we have the opportunity to\nutilize that data to generate documentation.  pyramid-marshmallow includes an\nutility that uses [apispec](https://apispec.readthedocs.io/en/stable/) to\ngenerate an [OpenAPI](https://swagger.io/resources/open-api/) specification for\nyour application.\n\nFirst, you\'ll need to install some extra dependencies.\n\n```bash\npip install pyramid-marshmallow[openapi]\n```\n\nNow you can generate your spec by simply passing in an ini file.\npyramid-marshmallow needs to run your application in order to inspect it, so\nthe ini file should contain all the necessary configuration to do so.\n\n```bash\ngenerate-spec development.ini\n```\n\nThis will output the spec to stdout as JSON.  You can set the `--output` flag\nto output the results to a file.\n\nYou can set `--format yaml` to output the spec as YAML instead or\n`--format html` to output the spec as an HTML file, powered by\n[ReDoc](https://github.com/Redocly/redoc).\n\nBy default, your spec will be titled "Untitled" and versioned "0.1.0".  You can\nchange this by setting `openapi.title` and `openapi.version` in your ini file.\n\n### Documenting Your API\n\nDocumentation will be autogenerated from the structure of your Pyramid app and\nyour `validate` and `marshal` declarations.  You can document schema fields by\nsetting the `description` property.\n\n```python\nclass Hello(Schema):\n    name = String(required=True, description=\'Your first and last name.\')\n```\n\nThe first line of a view callable\'s docstring will be used as the `summary`\nproperty.  The following lines will be used for the `description`.\n\n\nDocumentation for the endpoint will be pulled from the view callable\'s\ndocstring.  The first line becomes the `summary` and the remaining lines become\nthe `description`.\n\nYou can also a line with three hyphens followed by a YAML object.  This will be\nmerged with the auto-generated\n[operation object](https://swagger.io/specification/#operationObject) and can\nbe used to manually augment the spec.\n\n```python\n@view_config(\n    context=WidgetResource,\n    method=\'post\',\n    validate=WidgetSchema(),\n)\ndef create_widget(context, request):\n    """\n    Create new widget\n\n    Creates a new widget with an attached plumbus.\n\n    ---\n    responses:\n        201:\n            description: Indicates the widget was successfully created.\n    """\n    widget.make()\n    return HTTPCreated()\n```\n\nYou can also pass a dictionary as the `api_spec` property to\n`Configurator.add_view` or `@view_config`, which will be merged with the spec\nin the same way.  This can be advantageous if a single function services\nmultiple views.\n\n```python\n@view_config(\n    context=WidgetResource,\n    method=\'post\',\n    validate=WidgetSchema(),\n    api_spec={\n        \'summary\': \'Create widget\',\n    },\n)\n@view_config(\n    context=WidgetResource,\n    method=\'put\',\n    validate=WidgetSchema(),\n    api_spec={\n        \'summary\': \'Update widget\',\n    },\n)\ndef create_widget(context, request):\n    """\n    Create/update new widget\n\n    ---\n    responses:\n        201:\n            description: Indicates the widget was successfully created.\n    """\n    widget.create_or_update()\n    return HTTPCreated()\n```\n\n## URL Traversal\n\nIf you\'re using Pyramid\'s URL traversal, the generated spec may be mostly\nempty.  This is because pyramid-marshmallow has no way of knowing where in the\nresource tree a resource is.  You can denote this by setting the `__path__`\nproperty on each resource.\n\n```python\nclass Widget(Resource):\n    __path__ = \'/widget\'\n```\n\nViews attached to this resource will then be added to the spec.\n\nYou can add parameters to your path via the `__params__` property.  You can\nalso tag all attached views via `__tag__`.  Once you define a tag in one\nresource, you can use it elsewhere by setting `__tag__` to the tag name.\n\n```python\nclass Widget(Resource):\n    __path__ = \'/widget/{widgetId}\'\n    __params__ = [{\n        \'name\': \'widgetId\',\n        \'schema\': {\n            \'type\': \'integer\',\n        },\n    }]\n    __tag__ = {\n        \'name\': \'widgets\',\n        \'description\': \'Endpoints for managing a widget.\',\n    }\n```\n\n## Mergefile\n\nYou likely will wish to augment your API spec with a description, additional\ncomponents, and more.  This can be achieved with a mergefile.  Write a YAML\nfile with the parts of the spec you wish to augment and pass the filename as\nthe `--merge` flag with `generate-spec`.  The file will be loaded and merged\ninto the generated spec.\n\n## Zones\n\nIt may be that not all endpoints are made available to all users.  For example,\nyou may have all endpoints available internally but only select ones available\npublicly.  You would then want separate API docs for internal users versus\nexternal users.  This can be achieved using zones.\n\nTag each endpoint by setting `api_zone` in `Configuration.add_view` or\n`@view_config`.  By default, all endpoints regardless of zone will be added to\nthe spec.  Set the `--zone` flag in `generate-spec` to only put endpoints\nassigned to that zone in the spec.\n\n## Prior Art\n\n[pyramid-apispec](https://pypi.org/project/pyramid-apispec/) allows you to\naugment view callable docstrings with OpenAPI definitions and can reference\nMarshmallow schemas with the apispec Marshmallow plugin.  It does not support\nvalidating input and marshalling output.  Schemas and routes must be manually\ndeclared.\n\n[Cornice](https://cornice.readthedocs.io/en/latest/schema.html#using-marshmallow)\nsupports validation with Marshmallow schemas, however only on Cornice\nresources, not arbitrary Pyramid endpoints.  It does not support\nauto-generating OpenAPI documentation.\n',
    'author': 'Theron Luhn',
    'author_email': 'theron@luhn.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/luhn/pyramid-marshmallow',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
